local _G = GLOBAL
local _T = _G.TUNING

local healthDegenerationInterval = GetModConfigData("HealthDegenerationInterval") -- Interval between degenerating lost health
local healthDegeneratedPerInterval = GetModConfigData("HealthDegeneratedPerInterval") -- Amount of health degenerated per chosen interval
local healthDegenerationThreshold = GetModConfigData("HealthDegenerationThreshold") / 100 -- Only degenerate health, if it is not lower than this percentage.
local hungerThresholdForDegeneratingHealth = GetModConfigData("HungerThresholdForDegeneratingHealth") / 100
local sanityThresholdForDegeneratingHealth = GetModConfigData("SanityThresholdForDegeneratingHealth") / 100
local healthDegenerationAllThresholdsMustBeBroken = GetModConfigData("HealthDegenerationAllThresholdsMustBeBroken")
local healthDegenerationRemoveHungerHealthDegen = GetModConfigData("HealthDegenerationRemoveHungerHealthDegen")
local healthDegenerationCanKill = GetModConfigData("HealthDegenerationCanKill")

local healthRegenerationInterval = GetModConfigData("HealthRegenerationInterval") -- Interval between regenerating lost health
local healthRegeneratedPerInterval = GetModConfigData("HealthRegeneratedPerInterval") -- Amount of health regenerated per chosen interval
local healthRegenerationThreshold = GetModConfigData("HealthRegenerationThreshold") / 100 -- Only regenerate health, if it is not higher than this percentage.
local delayAfterMovingBeforePlayerCanRegenerateHealth = GetModConfigData("DelayAfterMovingBeforePlayerCanRegenerateHealth")
local delayAfterFightingBeforePlayerCanRegenerateHealth = GetModConfigData("DelayAfterFightingBeforePlayerCanRegenerateHealth")
local hungerRequirementForRegeneratingHealth = GetModConfigData("HungerRequirementForRegeneratingHealth") / 100
local sanityRequirementForRegeneratingHealth = GetModConfigData("SanityRequirementForRegeneratingHealth") / 100
local healthRegenerationHungerCost = GetModConfigData("HealthRegenerationHungerCost")
local healthRegenerationSanityCost = GetModConfigData("HealthRegenerationSanityCost")

local sanityDegenerationInterval = GetModConfigData("SanityDegenerationInterval") -- Interval between degenerating lost sanity
local sanityDegeneratedPerInterval = GetModConfigData("SanityDegeneratedPerInterval") -- Amount of sanity degenerated per chosen interval
local sanityDegenerationThreshold = GetModConfigData("SanityDegenerationThreshold") / 100 -- Only degenerate sanity, if it is not lower than this percentage.
local hungerThresholdForDegeneratingSanity = GetModConfigData("HungerThresholdForDegeneratingSanity") / 100
local healthThresholdForDegeneratingSanity = GetModConfigData("HealthThresholdForDegeneratingSanity") / 100
local sanityDegenerationAllThresholdsMustBeBroken = GetModConfigData("SanityDegenerationAllThresholdsMustBeBroken")

local sanityRegenerationInterval = GetModConfigData("SanityRegenerationInterval") -- Interval between regenerating lost sanity
local sanityRegeneratedPerInterval = GetModConfigData("SanityRegeneratedPerInterval") -- Amount of sanity regenerated per chosen interval
local sanityRegenerationThreshold = GetModConfigData("SanityRegenerationThreshold") / 100 -- Only regenerate sanity, if it is not higher than this percentage.
local delayAfterMovingBeforePlayerCanRegenerateSanity = GetModConfigData("DelayAfterMovingBeforePlayerCanRegenerateSanity")
local delayAfterFightingBeforePlayerCanRegenerateSanity = GetModConfigData("DelayAfterFightingBeforePlayerCanRegenerateSanity")
local hungerRequirementForRegeneratingSanity = GetModConfigData("HungerRequirementForRegeneratingSanity") / 100
local healthRequirementForRegeneratingSanity = GetModConfigData("HealthRequirementForRegeneratingSanity") / 100
local sanityRegenerationHungerCost = GetModConfigData("SanityRegenerationHungerCost")
local sanityRegenerationHealthCost = GetModConfigData("SanityRegenerationHealthCost")

local hungerDegenerationInterval = GetModConfigData("HungerDegenerationInterval") -- Interval between degenerating lost hunger
local hungerDegeneratedPerInterval = GetModConfigData("HungerDegeneratedPerInterval") -- Amount of hunger degenerated per chosen interval
local hungerDegenerationThreshold = GetModConfigData("HungerDegenerationThreshold") / 100 -- Only degenerate hunger, if it is not lower than this percentage.
local sanityThresholdForDegeneratingHunger = GetModConfigData("SanityThresholdForDegeneratingHunger") / 100
local healthThresholdForDegeneratingHunger = GetModConfigData("HealthThresholdForDegeneratingHunger") / 100
local hungerDegenerationAllThresholdsMustBeBroken = GetModConfigData("HungerDegenerationAllThresholdsMustBeBroken")

local hungerRegenerationInterval = GetModConfigData("HungerRegenerationInterval") -- Interval between regenerating lost hunger
local hungerRegeneratedPerInterval = GetModConfigData("HungerRegeneratedPerInterval") -- Amount of hunger regenerated per chosen interval
local hungerRegenerationThreshold = GetModConfigData("HungerRegenerationThreshold") / 100 -- Only regenerate hunger, if it is not higher than this percentage.
local delayAfterMovingBeforePlayerCanRegenerateHunger = GetModConfigData("DelayAfterMovingBeforePlayerCanRegenerateHunger")
local delayAfterFightingBeforePlayerCanRegenerateHunger = GetModConfigData("DelayAfterFightingBeforePlayerCanRegenerateHunger")
local sanityRequirementForRegeneratingHunger = GetModConfigData("SanityRequirementForRegeneratingHunger") / 100
local healthRequirementForRegeneratingHunger = GetModConfigData("HealthRequirementForRegeneratingHunger") / 100
local hungerRegenerationHealthCost = GetModConfigData("HungerRegenerationHealthCost")
local hungerRegenerationSanityCost = GetModConfigData("HungerRegenerationSanityCost")

local function roundToNthDecimal(num, n)
  local mult = 10^(n or 0)
  return math.floor(num * mult + 0.5) / mult
end

local function fuzzyEquals( x, y, delta )
    return x == y or math.abs( x - y ) <= ( delta or .0001 )
end

local function checkXHigherThanOrEqualToY( x, y, delta )
    return x == y or math.abs( x - y ) <= ( delta or .0001 ) or x > y
end

local function checkXLowerThanOrEqualToY( x, y, delta )
    return x == y or math.abs( x - y ) <= ( delta or .0001 ) or x < y
end

local function onFighting(inst)
	-- print("Fight registered")
	inst.SRSLastFightTime = _G.GetTime()
end

local updateInterval = _T.SEG_TIME / 16

local GetPercentWithPenalty = function(self)
	return self.currenthealth / self:GetMaxHealth()
end

AddPlayerPostInit(function(inst)
	-- For testing purposes!
	-- inst.components.health:SetPercent(0.1)
	-- inst.components.sanity:SetPercent(0.5, false)
	-- inst.components.hunger:SetPercent(0.5, false)
	-- For testing purposes!
	
	inst.SRSHealthDegenOn = inst.components.health and healthDegenerationInterval ~= 0 and healthDegeneratedPerInterval ~= 0
	inst.SRSHealthRegenOn = inst.components.health and healthRegenerationInterval ~= 0 and healthRegeneratedPerInterval ~= 0
	inst.SRSSanityDegenOn = inst.components.sanity and sanityDegenerationInterval ~= 0 and sanityDegeneratedPerInterval ~= 0
	inst.SRSSanityRegenOn = inst.components.sanity and sanityRegenerationInterval ~= 0 and sanityRegeneratedPerInterval ~= 0
	inst.SRSHungerDegenOn = inst.components.hunger and hungerDegenerationInterval ~= 0 and hungerDegeneratedPerInterval ~= 0
	inst.SRSHungerRegenOn = inst.components.hunger and hungerRegenerationInterval ~= 0 and hungerRegeneratedPerInterval ~= 0
	
	-- print("SRSHealthDegenOn: "..tostring(inst.SRSHealthDegenOn))
	-- print("SRSHealthRegenOn: "..tostring(inst.SRSHealthRegenOn))
	-- print("SRSSanityDegenOn: "..tostring(inst.SRSSanityDegenOn))
	-- print("SRSSanityRegenOn: "..tostring(inst.SRSSanityRegenOn))
	-- print("SRSHungerDegenOn: "..tostring(inst.SRSHungerDegenOn))
	-- print("SRSHungerRegenOn: "..tostring(inst.SRSHungerRegenOn))
	
	inst.SRSCurrentTime = _G.GetTime()
	inst.SRSLastFightTime = -100
	inst.SRSLastMovementTime = -100
	
	if inst.components.health then
		inst.components.health.GetPercentWithPenalty = GetPercentWithPenalty
	end
	
	if inst.components.hunger and healthDegenerationRemoveHungerHealthDegen then
		local oldDoDec = inst.components.hunger.DoDec
		if oldDoDec then
			inst.components.hunger.DoDec = function(self, dt, ignore_damage, ...)
				oldDoDec(self, dt, true, ...)
			end
		end
	end
	
	if inst.components.combat then
		inst:ListenForEvent("attacked", onFighting)
		inst:ListenForEvent("blocked", onFighting)
		inst:ListenForEvent("onattackother", onFighting)
	end
	
	-- Do periodic regeneration
	inst:DoPeriodicTask(updateInterval, function(inst)
		-- print("----------------------------------------------------------")
		
		-- If player is dead, don't check anything else, and reset.
		if inst.components.health and inst.components.health:IsDead() then
			inst.SRSHealthDegenerationQueued = false
			inst.SRSHealthRegenerationQueued = false
			inst.SRSSanityDegenerationQueued = false
			inst.SRSSanityRegenerationQueued = false
			inst.SRSHungerDegenerationQueued = false
			inst.SRSHungerRegenerationQueued = false
			-- print("Player is dead")
			return
		end
		
		local health = inst.components.health
		local sanity = inst.components.sanity
		local hunger = inst.components.hunger
		
		inst.SRSCurrentTime = _G.GetTime()
		inst.SRSTimeSinceMovement = inst.SRSCurrentTime - inst.SRSLastMovementTime
		inst.SRSTimeSinceFighting = inst.SRSCurrentTime - inst.SRSLastFightTime
		
		if inst.Physics:GetMotorSpeed() ~= 0 then
			inst.SRSLastMovementTime = inst.SRSCurrentTime
		end
		
		-----------------------------------------------------------------------------------------------------------------
		
		-- Now comes the checks for whether regen/degen should occur, and these ARE done every 1/16 of a segment, for each regen/degen.
		-- It may seem weird to do all these checks every 1/16 of a segment, but it makes sense. If a regen is queued, we need to periodically check
		-- if it should be cancelled (if the player no longer reaches the requirements). If always let it run until the wait is over and THEN
		-- cancel it, then if the player reaches the requirements before that queue is over, they will get the regen, even though they were outside
		-- the requirements while waiting for the regen.
		-- And since we need to do all these checks anyway, to see if the player reaches the requirements when trying to queue the next regen,
		-- we might as well just have those checks be done once every sixteenth, right here at the beginning.
			
		-----------------------------------------------------------------------------------------------------------------
		
		if inst.SRSHealthDegenOn and health then
			-- print("Health degen: Has 0 health: "..tostring(fuzzyEquals(health.currenthealth, 0, 0.001)))
			-- print("Health degen: Current health: "..roundToNthDecimal(health.currenthealth, 3).."/"..roundToNthDecimal(health:GetMaxHealth(), 3))
			-- print("Health degen: Threshold: "..roundToNthDecimal(healthDegenerationThreshold, 3).."% or "..roundToNthDecimal(health:GetMaxHealth() * healthDegenerationThreshold, 3))
			-- print("Health degen: Threshold check passing: "..tostring(not (healthDegenerationThreshold ~= 0 and checkXLowerThanOrEqualToY(health.currenthealth, health:GetMaxHealth() * healthDegenerationThreshold))))
			-- print("Health degen: Hunger check passing: "..tostring(not (hungerThresholdForDegeneratingHealth < 1 and hunger and hunger:GetPercent() > hungerThresholdForDegeneratingHealth)))
			-- print("Health degen: Sanity check passing: "..tostring(not (sanityThresholdForDegeneratingHealth < 1 and sanity and sanity:GetPercent(true) > sanityThresholdForDegeneratingHealth)))
			
			local healthDegenShouldBeQueued = true
			-- Do all requirement checks for health degeneration.
			
			-- Cancel degen if the player is already below 1.5 health, unless we allow health degen to kill.
			if (not healthDegenerationCanKill and health.currenthealth < 1.5)
			-- Cancel degeneration, if the threshold is not 0% and the current health is below or equal to the threshold.
			or (healthDegenerationThreshold ~= 0 and checkXLowerThanOrEqualToY(health.currenthealth, health:GetMaxHealth() * healthDegenerationThreshold))
			or (healthDegenerationAllThresholdsMustBeBroken
				-- If both are 0 (never broken), we consider the thresholds turned off, and return false, so we always run the degen.
				and (not (hungerThresholdForDegeneratingHealth == 0 and sanityThresholdForDegeneratingHealth == 0))
				and
				(
				-- Do degen (return false) if ALL thresholds are violated.
				-- Cancel degen (return true) if JUST ONE threshold is NOT violated.
				-- If threshold is 0, the whole check is false, since it is effectively never broken, and should not be considered.
				-- We have already done the check for ALL thresholds being 0, so there is at least one active threshold.
				(hungerThresholdForDegeneratingHealth > 0 and hunger and hunger:GetPercent() > hungerThresholdForDegeneratingHealth)
				or
				(sanityThresholdForDegeneratingHealth > 0 and sanity and sanity:GetPercent(true) > sanityThresholdForDegeneratingHealth)
				)
			)
			or (not healthDegenerationAllThresholdsMustBeBroken
				-- If both are 0 (never broken), we consider the thresholds turned off, and return false, so we always run the degen.
				and (not (hungerThresholdForDegeneratingHealth == 0 and sanityThresholdForDegeneratingHealth == 0))
				and
				(
				-- Do degen (return false) if ANY threshold is violated.
				-- Cancel degen (return true) if NO thresholds are violated.
				-- If threshold is 0, it is effectively never broken, which amounts to a "true".
				-- We have already done the check for ALL thresholds being 0, so there is at least one active threshold.
				(hungerThresholdForDegeneratingHealth == 0 or hunger and hunger:GetPercent() > hungerThresholdForDegeneratingHealth)
				and
				(sanityThresholdForDegeneratingHealth == 0 or sanity and sanity:GetPercent(true) > sanityThresholdForDegeneratingHealth)
				)
			)
				then
				-- If anything fails, cancel queued health degeneration (if any).
				inst.SRSHealthDegenerationQueued = false
				healthDegenShouldBeQueued = false
				-- print("Health degen: Invalid state")
			else
				if inst.SRSHealthDegenerationQueued then
					inst.SRSHealthDegenerationSixteenthsToWait = inst.SRSHealthDegenerationSixteenthsToWait - 1
					-- print("Health degen: Waiting another "..inst.SRSHealthDegenerationSixteenthsToWait.." sixteenths...")
					if checkXLowerThanOrEqualToY(inst.SRSHealthDegenerationSixteenthsToWait, 0) then
						local healthChange = healthDegeneratedPerInterval
						if health.currenthealth - healthChange < 0 then
							healthChange = health.currenthealth - (healthDegenerationCanKill and 0 or 1)
						end
						
						-- print("Health degen: Applying change => current health: "..roundToNthDecimal(health.currenthealth, 3).." || proposed change: "..roundToNthDecimal(healthDegeneratedPerInterval, 3).." || thresholded change: "..roundToNthDecimal(healthChange, 3).." || % of proposed change: "..roundToNthDecimal(healthChange / healthDegeneratedPerInterval, 3))
						
						health:DoDelta(-healthChange, true)
						inst.SRSHealthDegenerationQueued = false
						-- print("Health degen: Result: "..health.currenthealth)
					end
				end
				
				if not inst.SRSHealthDegenerationQueued and healthDegenShouldBeQueued then
					inst.SRSHealthDegenerationQueued = true
					inst.SRSHealthDegenerationSixteenthsToWait = 
						healthDegenerationInterval == 1 and 1 or
						healthDegenerationInterval == 2 and 2 or
						healthDegenerationInterval == 3 and 4 or
						healthDegenerationInterval == 4 and 8 or
						healthDegenerationInterval == 5 and 16 or
						healthDegenerationInterval == 6 and 32 or
						healthDegenerationInterval == 7 and 48 or
						healthDegenerationInterval == 8 and 64 or
						healthDegenerationInterval == 9 and 80 or
						healthDegenerationInterval == 10 and 96 or
						healthDegenerationInterval == 11 and 112 or
						healthDegenerationInterval == 12 and 128 or
						healthDegenerationInterval == 13 and 144 or
						healthDegenerationInterval == 14 and 160 or
						healthDegenerationInterval == 15 and 176 or
						healthDegenerationInterval == 16 and 192 or
						healthDegenerationInterval == 17 and 208 or
						healthDegenerationInterval == 18 and 224 or
						healthDegenerationInterval == 19 and 240 or
						healthDegenerationInterval == 20 and 256 or
						healthDegenerationInterval == 21 and 272 or
						healthDegenerationInterval == 22 and 288 or
						healthDegenerationInterval == 23 and 304 or
						healthDegenerationInterval == 24 and 320 or
						healthDegenerationInterval == 25 and 336 or
						healthDegenerationInterval == 26 and 352
					-- print("Health degen: Degeneration queued. Waiting "..inst.SRSHealthDegenerationSixteenthsToWait.." sixteenths.")
				end
			end
		end
		
		-----------------------------------------------------------------------------------------------------------------
		
		if inst.SRSHealthRegenOn and health then
			-- print("Health regen: Has full health: "..tostring(fuzzyEquals(health.currenthealth, health:GetMaxHealth(), 0.001)))
			-- print("Health regen: Current health: "..roundToNthDecimal(health.currenthealth, 3).."/"..roundToNthDecimal(health:GetMaxHealth(), 3))
			-- print("Health regen: Threshold: "..roundToNthDecimal(healthRegenerationThreshold, 3).."% or "..roundToNthDecimal(health:GetMaxHealth() * healthRegenerationThreshold, 3))
			-- print("Health regen: Threshold check passing: "..tostring(not (healthRegenerationThreshold ~= 1 and checkXHigherThanOrEqualToY(health.currenthealth, health:GetMaxHealth() * healthRegenerationThreshold))))
			-- print("Health regen: Movement check passing: "..tostring(not (delayAfterMovingBeforePlayerCanRegenerateHealth > 0 and (inst.SRSTimeSinceMovement < delayAfterMovingBeforePlayerCanRegenerateHealth))))
			-- print("Health regen: Fight check passing: "..tostring(not (delayAfterFightingBeforePlayerCanRegenerateHealth > 0 and (inst.SRSTimeSinceFighting < delayAfterFightingBeforePlayerCanRegenerateHealth))))
			-- print("Health regen: Hunger check passing: "..tostring(not (hungerRequirementForRegeneratingHealth > 0 and hunger and hunger:GetPercent() < hungerRequirementForRegeneratingHealth)))
			-- print("Health regen: Sanity check passing: "..tostring(not (sanityRequirementForRegeneratingHealth > 0 and sanity and sanity:GetPercent(true) < sanityRequirementForRegeneratingHealth)))
			
			-- Do all requirement checks for health regeneration.
			
			-- Cancel regen if health DEgen is queued.
			if inst.SRSHealthDegenerationQueued
			-- Cancel regen if the player is already at full health.
			or fuzzyEquals(health.currenthealth, health:GetMaxHealth(), 0.001)
			-- Cancel regeneration, if the threshold is not 100% and the current health is above or equal to the threshold.
			or (healthRegenerationThreshold ~= 1 and checkXHigherThanOrEqualToY(health.currenthealth, health:GetMaxHealth() * healthRegenerationThreshold))
			-- If the player cannot regenerate health while moving, check the locomotor and/or physics for movement.
			or (delayAfterMovingBeforePlayerCanRegenerateHealth > 0 and (inst.SRSTimeSinceMovement < delayAfterMovingBeforePlayerCanRegenerateHealth))
			-- If the player cannot regenerate health while fighting, check if we have waited long enough since last fighting.
			or (delayAfterFightingBeforePlayerCanRegenerateHealth > 0 and (inst.SRSTimeSinceFighting < delayAfterFightingBeforePlayerCanRegenerateHealth))
			-- If there is a hunger requirement, check if the player's hunger supports regenerating health.
			or (hungerRequirementForRegeneratingHealth > 0 and hunger and hunger:GetPercent() < hungerRequirementForRegeneratingHealth)
			-- If there is a sanity requirement, check if the player's sanity supports regenerating health.
			or (sanityRequirementForRegeneratingHealth > 0 and sanity and sanity:GetPercent(true) < sanityRequirementForRegeneratingHealth)
				then
				-- If anything fails, cancel queued health regeneration (if any).
				inst.SRSHealthRegenerationQueued = false
				-- print("Health regen: Invalid state")
			else
				if inst.SRSHealthRegenerationQueued then
					-- Count down sixteenths wait-count.
					inst.SRSHealthRegenerationSixteenthsToWait = inst.SRSHealthRegenerationSixteenthsToWait - 1
					-- print("Health regen: Waiting another "..inst.SRSHealthRegenerationSixteenthsToWait.." sixteenths...")
					-- If waiting time is over, do health regeneration.
					if checkXLowerThanOrEqualToY(inst.SRSHealthRegenerationSixteenthsToWait, 0) then
						local healthChange = healthRegeneratedPerInterval
						if health.currenthealth + healthChange > health:GetMaxHealth() * healthRegenerationThreshold then
							healthChange = (health:GetMaxHealth() * healthRegenerationThreshold) - health.currenthealth
						end
						
						local percentageOfHealthChangeApplied = healthChange / healthRegeneratedPerInterval
						
						-- Cancel regen if flat stat costs are not all available.
						if hunger and hunger.current < healthRegenerationHungerCost * percentageOfHealthChangeApplied
						or sanity and sanity.current < healthRegenerationSanityCost * percentageOfHealthChangeApplied
						then
							-- print("Health regen: Not sufficient stats to regen health.")
							inst.SRSHealthRegenerationQueued = false
						else
							-- print("Health regen: Applying change => current health: "..roundToNthDecimal(health.currenthealth, 3).." || proposed change: "..roundToNthDecimal(healthRegeneratedPerInterval, 3).." || thresholded change: "..roundToNthDecimal(healthChange, 3).." || % of proposed change: "..roundToNthDecimal(percentageOfHealthChangeApplied, 3))
							
							health:DoDelta(healthChange, true)
							inst.SRSHealthRegenerationQueued = false
							-- print("Health regen: Result: "..health.currenthealth)
							
							if hunger then
								hunger:DoDelta(-healthRegenerationHungerCost * percentageOfHealthChangeApplied, true)
							end
							
							if sanity then
								sanity:DoDelta(-healthRegenerationSanityCost * percentageOfHealthChangeApplied, true)
							end
						end
					end
				end
				
				if not inst.SRSHealthRegenerationQueued and health.currenthealth < health:GetMaxHealth() then
					inst.SRSHealthRegenerationQueued = true
					inst.SRSHealthRegenerationSixteenthsToWait = 
						healthRegenerationInterval == 1 and 1 or
						healthRegenerationInterval == 2 and 2 or
						healthRegenerationInterval == 3 and 4 or
						healthRegenerationInterval == 4 and 8 or
						healthRegenerationInterval == 5 and 16 or
						healthRegenerationInterval == 6 and 32 or
						healthRegenerationInterval == 7 and 48 or
						healthRegenerationInterval == 8 and 64 or
						healthRegenerationInterval == 9 and 80 or
						healthRegenerationInterval == 10 and 96 or
						healthRegenerationInterval == 11 and 112 or
						healthRegenerationInterval == 12 and 128 or
						healthRegenerationInterval == 13 and 144 or
						healthRegenerationInterval == 14 and 160 or
						healthRegenerationInterval == 15 and 176 or
						healthRegenerationInterval == 16 and 192 or
						healthRegenerationInterval == 17 and 208 or
						healthRegenerationInterval == 18 and 224 or
						healthRegenerationInterval == 19 and 240 or
						healthRegenerationInterval == 20 and 256 or
						healthRegenerationInterval == 21 and 272 or
						healthRegenerationInterval == 22 and 288 or
						healthRegenerationInterval == 23 and 304 or
						healthRegenerationInterval == 24 and 320 or
						healthRegenerationInterval == 25 and 336 or
						healthRegenerationInterval == 26 and 352
					-- print("Health regen: Regeneration queued. Waiting "..inst.SRSHealthRegenerationSixteenthsToWait.." sixteenths.")
				end
			end
		end
		
		-----------------------------------------------------------------------------------------------------------------
		
		if inst.SRSSanityDegenOn and sanity then
			-- print("Sanity degen: Has 0 sanity: "..tostring(fuzzyEquals(sanity.current, 0, 0.001)))
			-- print("Sanity degen: Current sanity: "..roundToNthDecimal(sanity.current, 3).."/"..roundToNthDecimal(sanity:GetMaxSanity(), 3))
			-- print("Sanity degen: Threshold: "..roundToNthDecimal(sanityDegenerationThreshold, 3).."% or "..roundToNthDecimal(sanity:GetMaxSanity() * sanityDegenerationThreshold, 3))
			-- print("Sanity degen: Threshold check passing: "..tostring(not (sanityDegenerationThreshold ~= 0 and checkXLowerThanOrEqualToY(sanity.current, sanity:GetMaxSanity() * sanityDegenerationThreshold))))
			-- print("Sanity degen: Health check passing: "..tostring(not (healthThresholdForDegeneratingSanity < 1 and health and health:GetPercentWithPenalty() > healthThresholdForDegeneratingSanity)))
			-- print("Sanity degen: Hunger check passing: "..tostring(not (hungerThresholdForDegeneratingSanity < 1 and hunger and hunger:GetPercent() > hungerThresholdForDegeneratingSanity)))
			
			local sanityDegenShouldBeQueued = true
			-- Do all requirement checks for sanity degeneration.
			
			-- Cancel degen if the player is already at 0 sanity.
			if fuzzyEquals(sanity.current, 0, 0.001)
			-- Cancel degen if the player has induced insanity.
			or inst.components.sanity.inducedinsanity
			-- Cancel degeneration, if the threshold is not 0% and the current sanity is below or equal to the threshold.
			or (sanityDegenerationThreshold ~= 0 and checkXLowerThanOrEqualToY(sanity.current, sanity:GetMaxSanity() * sanityDegenerationThreshold))
			or (sanityDegenerationAllThresholdsMustBeBroken
				-- If both are 0 (never broken), we consider the thresholds turned off, and return false, so we always run the degen.
				and (not (healthThresholdForDegeneratingSanity == 0 and hungerThresholdForDegeneratingSanity == 0))
				and
				(
				-- Do degen (return false) if ALL thresholds are violated.
				-- Cancel degen (return true) if JUST ONE threshold is NOT violated.
				-- If threshold is 0, the whole check is false, since it is effectively never broken, and should not be considered.
				-- We have already done the check for ALL thresholds being 0, so there is at least one active threshold.
				(healthThresholdForDegeneratingSanity > 0 and health and health:GetPercentWithPenalty() > healthThresholdForDegeneratingSanity)
				or
				(hungerThresholdForDegeneratingSanity > 0 and hunger and hunger:GetPercent() > hungerThresholdForDegeneratingSanity)
				)
			)
			or (not sanityDegenerationAllThresholdsMustBeBroken
				-- If both are 0 (never broken), we consider the thresholds turned off, and return false, so we always run the degen.
				and (not (healthThresholdForDegeneratingSanity == 0 and hungerThresholdForDegeneratingSanity == 0))
				and
				(
				-- Do degen (return false) if ANY threshold is violated.
				-- Cancel degen (return true) if NO thresholds are violated.
				-- If threshold is 0, it is effectively never broken, which amounts to a "true".
				-- We have already done the check for ALL thresholds being 0, so there is at least one active threshold.
				(healthThresholdForDegeneratingSanity == 0 or health and health:GetPercentWithPenalty() > healthThresholdForDegeneratingSanity)
				and
				(hungerThresholdForDegeneratingSanity == 0 or hunger and hunger:GetPercent() > hungerThresholdForDegeneratingSanity)
				)
			)
				then
				-- If anything fails, cancel queued sanity degeneration (if any).
				inst.SRSSanityDegenerationQueued = false
				sanityDegenShouldBeQueued = false
				-- print("Sanity degen: Invalid state")
			else
				if inst.SRSSanityDegenerationQueued then
					inst.SRSSanityDegenerationSixteenthsToWait = inst.SRSSanityDegenerationSixteenthsToWait - 1
					-- print("Sanity degen: Waiting another "..inst.SRSSanityDegenerationSixteenthsToWait.." sixteenths...")
					if checkXLowerThanOrEqualToY(inst.SRSSanityDegenerationSixteenthsToWait, 0) then
						local sanityChange = sanityDegeneratedPerInterval
						if sanity.current - sanityChange < 0 then
							sanityChange = sanity.current
						end
						
						-- print("Sanity degen: Applying change => current sanity: "..roundToNthDecimal(sanity.current, 3).." || proposed change: "..roundToNthDecimal(sanityDegeneratedPerInterval, 3).." || thresholded change: "..roundToNthDecimal(sanityChange, 3).." || % of proposed change: "..roundToNthDecimal(sanityChange / sanityDegeneratedPerInterval, 3))
						
						sanity:DoDelta(-sanityChange, true)
						inst.SRSSanityDegenerationQueued = false
						-- print("Sanity degen: Result: "..sanity.current)
					end
				end
				
				if not inst.SRSSanityDegenerationQueued and sanityDegenShouldBeQueued then
					inst.SRSSanityDegenerationQueued = true
					inst.SRSSanityDegenerationSixteenthsToWait = 
						sanityDegenerationInterval == 1 and 1 or
						sanityDegenerationInterval == 2 and 2 or
						sanityDegenerationInterval == 3 and 4 or
						sanityDegenerationInterval == 4 and 8 or
						sanityDegenerationInterval == 5 and 16 or
						sanityDegenerationInterval == 6 and 32 or
						sanityDegenerationInterval == 7 and 48 or
						sanityDegenerationInterval == 8 and 64 or
						sanityDegenerationInterval == 9 and 80 or
						sanityDegenerationInterval == 10 and 96 or
						sanityDegenerationInterval == 11 and 112 or
						sanityDegenerationInterval == 12 and 128 or
						sanityDegenerationInterval == 13 and 144 or
						sanityDegenerationInterval == 14 and 160 or
						sanityDegenerationInterval == 15 and 176 or
						sanityDegenerationInterval == 16 and 192 or
						sanityDegenerationInterval == 17 and 208 or
						sanityDegenerationInterval == 18 and 224 or
						sanityDegenerationInterval == 19 and 240 or
						sanityDegenerationInterval == 20 and 256 or
						sanityDegenerationInterval == 21 and 272 or
						sanityDegenerationInterval == 22 and 288 or
						sanityDegenerationInterval == 23 and 304 or
						sanityDegenerationInterval == 24 and 320 or
						sanityDegenerationInterval == 25 and 336 or
						sanityDegenerationInterval == 26 and 352
					-- print("Sanity degen: Degeneration queued. Waiting "..inst.SRSSanityDegenerationSixteenthsToWait.." sixteenths.")
				end
			end
		end
		
		-----------------------------------------------------------------------------------------------------------------
		
		if inst.SRSSanityRegenOn and sanity then
			-- print("Sanity regen: Has full sanity: "..tostring(fuzzyEquals(sanity.current, sanity:GetMaxSanity(), 0.001)))
			-- print("Sanity regen: Current sanity: "..roundToNthDecimal(sanity.current, 3).."/"..roundToNthDecimal(sanity:GetMaxSanity(), 3))
			-- print("Sanity regen: Threshold: "..roundToNthDecimal(sanityRegenerationThreshold, 3).."% or "..roundToNthDecimal(sanity:GetMaxSanity() * sanityRegenerationThreshold, 3))
			-- print("Sanity regen: Threshold check passing: "..tostring(not (sanityRegenerationThreshold ~= 1 and checkXHigherThanOrEqualToY(sanity.current, sanity:GetMaxSanity() * sanityRegenerationThreshold))))
			-- print("Sanity regen: Movement check passing: "..tostring(not (delayAfterMovingBeforePlayerCanRegenerateSanity > 0 and (inst.SRSTimeSinceMovement < delayAfterMovingBeforePlayerCanRegenerateSanity))))
			-- print("Sanity regen: Fight check passing: "..tostring(not (delayAfterFightingBeforePlayerCanRegenerateSanity > 0 and (inst.SRSTimeSinceFighting < delayAfterFightingBeforePlayerCanRegenerateSanity))))
			-- print("Sanity regen: Health check passing: "..tostring(not (healthRequirementForRegeneratingSanity > 0 and health and health:GetPercentWithPenalty() < healthRequirementForRegeneratingSanity)))
			-- print("Sanity regen: Hunger check passing: "..tostring(not (hungerRequirementForRegeneratingSanity > 0 and hunger and hunger:GetPercent() < hungerRequirementForRegeneratingSanity)))
			
			-- Do all requirement checks for sanity regeneration.
			
			-- Cancel regen if sanity DEgen is queued.
			if inst.SRSSanityDegenerationQueued
			-- Cancel regen if the player has induced insanity.
			or inst.components.sanity.inducedinsanity
			-- Cancel regen if the player is already at full sanity.
			or fuzzyEquals(sanity.current, sanity:GetMaxSanity(), 0.001)
			-- Cancel regeneration, if the threshold is not 100% and the current sanity is above or equal to the threshold.
			or (sanityRegenerationThreshold ~= 1 and checkXHigherThanOrEqualToY(sanity.current, sanity:GetMaxSanity() * sanityRegenerationThreshold))
			-- If the player cannot regenerate sanity while moving, check the locomotor and/or physics for movement.
			or (delayAfterMovingBeforePlayerCanRegenerateSanity > 0 and (inst.SRSTimeSinceMovement < delayAfterMovingBeforePlayerCanRegenerateSanity))
			-- If the player cannot regenerate sanity while fighting, check if we have waited long enough since last fighting.
			or (delayAfterFightingBeforePlayerCanRegenerateSanity > 0 and (inst.SRSTimeSinceFighting < delayAfterFightingBeforePlayerCanRegenerateSanity))
			-- If there is a health requirement, check if the player's health supports regenerating sanity.
			or (healthRequirementForRegeneratingSanity > 0 and health and health:GetPercentWithPenalty() < healthRequirementForRegeneratingSanity)
			-- If there is a hunger requirement, check if the player's hunger supports regenerating sanity.
			or (hungerRequirementForRegeneratingSanity > 0 and hunger and hunger:GetPercent() < hungerRequirementForRegeneratingSanity)
				then
				-- If anything fails, cancel queued sanity regeneration (if any).
				inst.SRSSanityRegenerationQueued = false
				-- print("Sanity regen: Invalid state")
			else
				if inst.SRSSanityRegenerationQueued then
					inst.SRSSanityRegenerationSixteenthsToWait = inst.SRSSanityRegenerationSixteenthsToWait - 1
					-- print("Sanity regen: Waiting another "..inst.SRSSanityRegenerationSixteenthsToWait.." sixteenths...")
					if checkXLowerThanOrEqualToY(inst.SRSSanityRegenerationSixteenthsToWait, 0) then
						local sanityChange = sanityRegeneratedPerInterval
						if sanity.current + sanityChange > sanity:GetMaxSanity() * sanityRegenerationThreshold then
							sanityChange = (sanity:GetMaxSanity() * sanityRegenerationThreshold) - sanity.current
						end
						
						local percentageOfSanityChangeApplied = sanityChange / sanityRegeneratedPerInterval
						
						-- Cancel regen if flat stat costs are not all available.
						if hunger and hunger.current < sanityRegenerationHungerCost * percentageOfSanityChangeApplied
						or health and health.currenthealth < sanityRegenerationHealthCost * percentageOfSanityChangeApplied
						then
							-- print("Sanity regen: Not sufficient stats to regen sanity.")
							inst.SRSSanityRegenerationQueued = false
						else
							-- print("Sanity regen: Applying change => current sanity: "..roundToNthDecimal(sanity.current, 3).." || proposed change: "..roundToNthDecimal(sanityRegeneratedPerInterval, 3).." || thresholded change: "..roundToNthDecimal(sanityChange, 3).." || % of proposed change: "..roundToNthDecimal(percentageOfSanityChangeApplied, 3))
							
							sanity:DoDelta(sanityChange, true)
							inst.SRSSanityRegenerationQueued = false
							-- print("Sanity regen: Result: "..sanity.current)
							if health then
								health:DoDelta(-sanityRegenerationHealthCost * percentageOfSanityChangeApplied, true)
							end
							
							if hunger then
								hunger:DoDelta(-sanityRegenerationHungerCost * percentageOfSanityChangeApplied, true)
							end
						end
					end
				end
				
				if not inst.SRSSanityRegenerationQueued and sanity.current < sanity:GetMaxSanity() then
					inst.SRSSanityRegenerationQueued = true
					inst.SRSSanityRegenerationSixteenthsToWait = 
						sanityRegenerationInterval == 1 and 1 or
						sanityRegenerationInterval == 2 and 2 or
						sanityRegenerationInterval == 3 and 4 or
						sanityRegenerationInterval == 4 and 8 or
						sanityRegenerationInterval == 5 and 16 or
						sanityRegenerationInterval == 6 and 32 or
						sanityRegenerationInterval == 7 and 48 or
						sanityRegenerationInterval == 8 and 64 or
						sanityRegenerationInterval == 9 and 80 or
						sanityRegenerationInterval == 10 and 96 or
						sanityRegenerationInterval == 11 and 112 or
						sanityRegenerationInterval == 12 and 128 or
						sanityRegenerationInterval == 13 and 144 or
						sanityRegenerationInterval == 14 and 160 or
						sanityRegenerationInterval == 15 and 176 or
						sanityRegenerationInterval == 16 and 192 or
						sanityRegenerationInterval == 17 and 208 or
						sanityRegenerationInterval == 18 and 224 or
						sanityRegenerationInterval == 19 and 240 or
						sanityRegenerationInterval == 20 and 256 or
						sanityRegenerationInterval == 21 and 272 or
						sanityRegenerationInterval == 22 and 288 or
						sanityRegenerationInterval == 23 and 304 or
						sanityRegenerationInterval == 24 and 320 or
						sanityRegenerationInterval == 25 and 336 or
						sanityRegenerationInterval == 26 and 352
					-- print("Sanity regen: Regeneration queued. Waiting "..inst.SRSSanityRegenerationSixteenthsToWait.." sixteenths.")
				end
			end
		end
		
		-----------------------------------------------------------------------------------------------------------------
		
		if inst.SRSHungerDegenOn and hunger then
			-- print("Hunger degen: Has 0 hunger: "..tostring(fuzzyEquals(hunger.current, 0, 0.001)))
			-- print("Hunger degen: Current hunger: "..roundToNthDecimal(hunger.current, 3).."/"..roundToNthDecimal(hunger.max, 3))
			-- print("Hunger degen: Threshold: "..roundToNthDecimal(hungerDegenerationThreshold, 3).."% or "..roundToNthDecimal(hunger.max * hungerDegenerationThreshold, 3))
			-- print("Hunger degen: Threshold check passing: "..tostring(not (hungerDegenerationThreshold ~= 0 and checkXLowerThanOrEqualToY(hunger.current, hunger.max * hungerDegenerationThreshold))))
			-- print("Hunger degen: Health check passing: "..tostring(not (healthThresholdForDegeneratingHunger < 1 and health and health:GetPercentWithPenalty() > healthThresholdForDegeneratingHunger)))
			-- print("Hunger degen: Sanity check passing: "..tostring(not (sanityThresholdForDegeneratingHunger < 1 and sanity and sanity:GetPercent(true) > sanityThresholdForDegeneratingHunger)))
			
			local hungerDegenShouldBeQueued = true
			-- Do all requirement checks for hunger degeneration.
			
			-- Cancel degen if the player is already at 0 hunger.
			if fuzzyEquals(hunger.current, 0, 0.001)
			-- Cancel degeneration, if the threshold is not 0% and the current hunger is below or equal to the threshold.
			or (hungerDegenerationThreshold ~= 0 and checkXLowerThanOrEqualToY(hunger.current, hunger.max * hungerDegenerationThreshold))
			or (hungerDegenerationAllThresholdsMustBeBroken
				-- If both are 0 (never broken), we consider the thresholds turned off, and return false, so we always run the degen.
				and (not (healthThresholdForDegeneratingHunger == 0 and sanityThresholdForDegeneratingHunger == 0))
				and
				(
				-- Do degen (return false) if ALL thresholds are violated.
				-- Cancel degen (return true) if JUST ONE threshold is NOT violated.
				-- If threshold is 0, the whole check is false, since it is effectively never broken, and should not be considered.
				-- We have already done the check for ALL thresholds being 0, so there is at least one active threshold.
				(healthThresholdForDegeneratingHunger > 0 and health and health:GetPercentWithPenalty() > healthThresholdForDegeneratingHunger)
				or
				(sanityThresholdForDegeneratingHunger > 0 and sanity and sanity:GetPercent(true) > sanityThresholdForDegeneratingHunger)
				)
			)
			or (not hungerDegenerationAllThresholdsMustBeBroken
				-- If both are 0 (never broken), we consider the thresholds turned off, and return false, so we always run the degen.
				and (not (healthThresholdForDegeneratingHunger == 0 and sanityThresholdForDegeneratingHunger == 0))
				and
				(
				-- Do degen (return false) if ANY threshold is violated.
				-- Cancel degen (return true) if NO thresholds are violated.
				-- If threshold is 0, it is effectively never broken, which amounts to a "true".
				-- We have already done the check for ALL thresholds being 0, so there is at least one active threshold.
				(healthThresholdForDegeneratingHunger == 0 or health and health:GetPercentWithPenalty() > healthThresholdForDegeneratingHunger)
				and
				(sanityThresholdForDegeneratingHunger == 0 or sanity and sanity:GetPercent(true) > sanityThresholdForDegeneratingHunger)
				)
			)
				then
				-- If anything fails, cancel queued hunger degeneration (if any).
				inst.SRSHungerDegenerationQueued = false
				hungerDegenShouldBeQueued = false
				-- print("Hunger degen: Invalid state")
			else
				if inst.SRSHungerDegenerationQueued then
					inst.SRSHungerDegenerationSixteenthsToWait = inst.SRSHungerDegenerationSixteenthsToWait - 1
					-- print("Hunger degen: Waiting another "..inst.SRSHungerDegenerationSixteenthsToWait.." sixteenths...")
					if checkXLowerThanOrEqualToY(inst.SRSHungerDegenerationSixteenthsToWait, 0) then
						local hungerChange = hungerDegeneratedPerInterval
						if hunger.current - hungerChange < 0 then
							hungerChange = hunger.current
						end
						
						-- print("Hunger degen: Applying change => current hunger: "..roundToNthDecimal(hunger.current, 3).." || proposed change: "..roundToNthDecimal(hungerDegeneratedPerInterval, 3).." || thresholded change: "..roundToNthDecimal(hungerChange, 3).." || % of proposed change: "..roundToNthDecimal(hungerChange / hungerDegeneratedPerInterval, 3))
						
						hunger:DoDelta(-hungerChange, true)
						inst.SRSHungerDegenerationQueued = false
						-- print("Hunger degen: Result: "..hunger.current)
					end
				end
				
				if not inst.SRSHungerDegenerationQueued and hungerDegenShouldBeQueued then
					inst.SRSHungerDegenerationQueued = true
					inst.SRSHungerDegenerationSixteenthsToWait = 
						hungerDegenerationInterval == 1 and 1 or
						hungerDegenerationInterval == 2 and 2 or
						hungerDegenerationInterval == 3 and 4 or
						hungerDegenerationInterval == 4 and 8 or
						hungerDegenerationInterval == 5 and 16 or
						hungerDegenerationInterval == 6 and 32 or
						hungerDegenerationInterval == 7 and 48 or
						hungerDegenerationInterval == 8 and 64 or
						hungerDegenerationInterval == 9 and 80 or
						hungerDegenerationInterval == 10 and 96 or
						hungerDegenerationInterval == 11 and 112 or
						hungerDegenerationInterval == 12 and 128 or
						hungerDegenerationInterval == 13 and 144 or
						hungerDegenerationInterval == 14 and 160 or
						hungerDegenerationInterval == 15 and 176 or
						hungerDegenerationInterval == 16 and 192 or
						hungerDegenerationInterval == 17 and 208 or
						hungerDegenerationInterval == 18 and 224 or
						hungerDegenerationInterval == 19 and 240 or
						hungerDegenerationInterval == 20 and 256 or
						hungerDegenerationInterval == 21 and 272 or
						hungerDegenerationInterval == 22 and 288 or
						hungerDegenerationInterval == 23 and 304 or
						hungerDegenerationInterval == 24 and 320 or
						hungerDegenerationInterval == 25 and 336 or
						hungerDegenerationInterval == 26 and 352
					-- print("Hunger degen: Degeneration queued. Waiting "..inst.SRSHungerDegenerationSixteenthsToWait.." sixteenths.")
				end
			end
		end
		
		-----------------------------------------------------------------------------------------------------------------
		
		if inst.SRSHungerRegenOn and hunger then
			-- print("Hunger regen: Has full hunger: "..tostring(fuzzyEquals(hunger.current, hunger.max, 0.001)))
			-- print("Hunger regen: Current hunger: "..roundToNthDecimal(hunger.current, 3).."/"..roundToNthDecimal(hunger.max, 3))
			-- print("Hunger regen: Threshold: "..roundToNthDecimal(hungerRegenerationThreshold, 3).."% or "..roundToNthDecimal(hunger.max * hungerRegenerationThreshold, 3))
			-- print("Hunger regen: Threshold check passing: "..tostring(not (hungerRegenerationThreshold ~= 1 and checkXHigherThanOrEqualToY(hunger.current, hunger.max * hungerRegenerationThreshold))))
			-- print("Hunger regen: Movement check passing: "..tostring(not (delayAfterMovingBeforePlayerCanRegenerateHunger > 0 and (inst.SRSTimeSinceMovement < delayAfterMovingBeforePlayerCanRegenerateHunger))))
			-- print("Hunger regen: Fight check passing: "..tostring(not (delayAfterFightingBeforePlayerCanRegenerateHunger > 0 and (inst.SRSTimeSinceFighting < delayAfterFightingBeforePlayerCanRegenerateHunger))))
			-- print("Hunger regen: Health check passing: "..tostring(not (healthRequirementForRegeneratingHunger > 0 and health and health:GetPercentWithPenalty() < healthRequirementForRegeneratingHunger)))
			-- print("Hunger regen: Sanity check passing: "..tostring(not (sanityRequirementForRegeneratingHunger > 0 and sanity and sanity:GetPercent(true) < sanityRequirementForRegeneratingHunger)))
			
			local hungerRegenShouldBeQueued = true
			-- Do all requirement checks for hunger regeneration.
			
			-- Cancel regen if hunger DEgen is queued.
			if inst.SRSHungerDegenerationQueued
			-- Cancel regen if the player is already at full hunger.
			or fuzzyEquals(hunger.current, hunger.max, 0.001)
			-- Cancel regeneration, if the threshold is not 100% and the current hunger is above or equal to the threshold.
			or (hungerRegenerationThreshold ~= 1 and hungerRegenerationThreshold < hunger:GetPercent())
			-- If the player cannot regenerate hunger while moving, check the locomotor and/or physics for movement.
			or (delayAfterMovingBeforePlayerCanRegenerateHunger > 0 and (inst.SRSTimeSinceMovement < delayAfterMovingBeforePlayerCanRegenerateHunger))
			-- If the player cannot regenerate hunger while fighting, check if we have waited long enough since last fighting.
			or (delayAfterFightingBeforePlayerCanRegenerateHunger > 0 and (inst.SRSTimeSinceFighting < delayAfterFightingBeforePlayerCanRegenerateHunger))
			-- If there is a health requirement, check if the player's health supports regenerating hunger.
			or (healthRequirementForRegeneratingHunger > 0 and health and health:GetPercentWithPenalty() < healthRequirementForRegeneratingHunger)
			-- If there is a sanity requirement, check if the player's hunger supports regenerating hunger.
			or (sanityRequirementForRegeneratingHunger > 0 and sanity and sanity:GetPercent(true) < sanityRequirementForRegeneratingHunger)
				then
				-- If anything fails, cancel queued hunger regeneration (if any).
				inst.SRSHungerRegenerationQueued = false
				hungerRegenShouldBeQueued = false
				-- print("Hunger regen: Invalid state")
			else
				if inst.SRSHungerRegenerationQueued then
					inst.SRSHungerRegenerationSixteenthsToWait = inst.SRSHungerRegenerationSixteenthsToWait - 1
					-- print("Hunger regen: Waiting another "..inst.SRSHungerRegenerationSixteenthsToWait.." sixteenths...")
					if checkXLowerThanOrEqualToY(inst.SRSHungerRegenerationSixteenthsToWait, 0) then
						local hungerChange = hungerRegeneratedPerInterval
						if hunger.current + hungerChange > hunger.max * hungerRegenerationThreshold then
							hungerChange = (hunger.max * hungerRegenerationThreshold) - hunger.current
						end
						
						local percentageOfHungerChangeApplied = hungerChange / hungerRegeneratedPerInterval
						
						-- Cancel regen if flat stat costs are not all available.
						if health and health.currenthealth < hungerRegenerationHealthCost * percentageOfHungerChangeApplied
						or sanity and sanity.current < hungerRegenerationSanityCost * percentageOfHungerChangeApplied
						then
							-- print("Hunger regen: Not sufficient stats to regen hunger.")
							inst.SRSHungerRegenerationQueued = false
						else
							-- print("Hunger regen: Applying change => current hunger: "..roundToNthDecimal(hunger.current, 3).." || proposed change: "..roundToNthDecimal(hungerRegeneratedPerInterval, 3).." || thresholded change: "..roundToNthDecimal(hungerChange, 3).." || % of proposed change: "..roundToNthDecimal(percentageOfHungerChangeApplied, 3))
							
							hunger:DoDelta(hungerChange, true)
							inst.SRSHungerRegenerationQueued = false
							-- print("Hunger regen: Result: "..hunger.current)
							
							if health then
								health:DoDelta(-hungerRegenerationHealthCost * percentageOfHungerChangeApplied, true)
							end
							
							if sanity then
								sanity:DoDelta(-hungerRegenerationSanityCost * percentageOfHungerChangeApplied, true)
							end
						end
					end
				end
				
				if not inst.SRSHungerRegenerationQueued and hungerRegenShouldBeQueued then
					inst.SRSHungerRegenerationQueued = true
					inst.SRSHungerRegenerationSixteenthsToWait = 
						hungerRegenerationInterval == 1 and 1 or
						hungerRegenerationInterval == 2 and 2 or
						hungerRegenerationInterval == 3 and 4 or
						hungerRegenerationInterval == 4 and 8 or
						hungerRegenerationInterval == 5 and 16 or
						hungerRegenerationInterval == 6 and 32 or
						hungerRegenerationInterval == 7 and 48 or
						hungerRegenerationInterval == 8 and 64 or
						hungerRegenerationInterval == 9 and 80 or
						hungerRegenerationInterval == 10 and 96 or
						hungerRegenerationInterval == 11 and 112 or
						hungerRegenerationInterval == 12 and 128 or
						hungerRegenerationInterval == 13 and 144 or
						hungerRegenerationInterval == 14 and 160 or
						hungerRegenerationInterval == 15 and 176 or
						hungerRegenerationInterval == 16 and 192 or
						hungerRegenerationInterval == 17 and 208 or
						hungerRegenerationInterval == 18 and 224 or
						hungerRegenerationInterval == 19 and 240 or
						hungerRegenerationInterval == 20 and 256 or
						hungerRegenerationInterval == 21 and 272 or
						hungerRegenerationInterval == 22 and 288 or
						hungerRegenerationInterval == 23 and 304 or
						hungerRegenerationInterval == 24 and 320 or
						hungerRegenerationInterval == 25 and 336 or
						hungerRegenerationInterval == 26 and 352
					-- print("Hunger regen: Regeneration queued. Waiting "..inst.SRSHungerRegenerationSixteenthsToWait.." sixteenths.")
				end
			end
		end
	end)
end)